/**
 * 'Higher Order Component' that controls the props of a wrapped
 * component via stores.
 *
 * Expects the Component to have two static methods:
 *   - getStores(): Should return an array of stores.
 *   - getPropsFromStores(props): Should return the props from the stores.
 *
 * Example using old React.createClass() style:
 *
 *    const MyComponent = React.createClass({
 *      statics: {
 *        getStores(props) {
 *          return [myStore]
 *        },
 *        getPropsFromStores(props) {
 *          return myStore.getState()
 *        }
 *      },
 *      render() {
 *        // Use this.props like normal ...
 *      }
 *    })
 *    MyComponent = connectToStores(MyComponent)
 *
 *
 * Example using ES6 Class:
 *
 *    class MyComponent extends React.Component {
 *      static getStores(props) {
 *        return [myStore]
 *      }
 *      static getPropsFromStores(props) {
 *        return myStore.getState()
 *      }
 *      render() {
 *        // Use this.props like normal ...
 *      }
 *    }
 *    MyComponent = connectToStores(MyComponent)
 *
 * A great explanation of the merits of higher order components can be found at
 * http://bit.ly/1abPkrP
 */

'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _functions = require('./functions');

function connectToStores(Component) {
  // Check for required static methods.
  if (!(0, _functions.isFunction)(Component.getStores)) {
    throw new Error('connectToStores() expects the wrapped component to have a static getStores() method');
  }
  if (!(0, _functions.isFunction)(Component.getPropsFromStores)) {
    throw new Error('connectToStores() expects the wrapped component to have a static getPropsFromStores() method');
  }

  // Wrapper Component.
  var StoreConnection = _react2['default'].createClass({
    displayName: 'StoreConnection',

    getInitialState: function getInitialState() {
      return Component.getPropsFromStores(this.props, this.context);
    },

    componentDidMount: function componentDidMount() {
      var _this = this;

      var stores = Component.getStores(this.props, this.context);
      stores.forEach(function (store) {
        store.listen(_this.onChange);
      });
      if (Component.componentDidConnect) {
        Component.componentDidConnect(this.props, this.context);
      }
    },

    componentWillUnmount: function componentWillUnmount() {
      var _this2 = this;

      var stores = Component.getStores(this.props, this.context);
      stores.forEach(function (store) {
        store.unlisten(_this2.onChange);
      });
    },

    onChange: function onChange() {
      this.setState(Component.getPropsFromStores(this.props, this.context));
    },

    render: function render() {
      return _react2['default'].createElement(Component, (0, _functions.assign)({}, this.props, this.state));
    }
  });

  return StoreConnection;
}

exports['default'] = connectToStores;
module.exports = exports['default'];